import { TeleBot } from "../src";
import { Message } from "../src/types/telegram";

const {
    TEST_TELEBOT_TOKEN: TOKEN,
    TEST_TELEBOT_USER: USER
} = process.env;

type Fn = (message: Message) => any[];

type MethodTestCase = {
    name: string;
    props: Fn | any[];
    // expect?: object;
    expect(message: Message, lastMessage?: Message): void;
};

const BOT_TEXT = "bot test message";
const BOT_PARAMS = {
    disable_notification: true
};

const METHODS: MethodTestCase[] = [
    {
        name: "sendMessage",
        props: [USER, BOT_TEXT, BOT_PARAMS],
        expect: ((message) => {
            expect(message).toHaveProperty("text", BOT_TEXT);
        })
    },
    {
        name: "forwardMessage",
        props: (lastMessage) => [USER, USER, lastMessage.message_id, BOT_PARAMS],
        expect: ((message) => {
            expect(message).toHaveProperty("text", BOT_TEXT);
        })
    }
];

if (!TOKEN || !USER) {
    throw new Error("Bot TEST_TELEBOT_TOKEN or TEST_TELEBOT_USER is not provided");
}

describe("TeleBot methods", () => {

    const bot = new TeleBot({
        token: TOKEN
    }) as any;

    const promise = Promise.resolve();

    METHODS.forEach((method: MethodTestCase) => {
        let lastResponse: Message;

        test(method.name, async function (done) {
            try {
                const props = Array.isArray(method.props) ? method.props : method.props.call(bot, lastResponse);
                const response = await bot[method.name].call(bot, ...props);

                // eslint-disable-next-line no-console
                console.log(response, lastResponse);

                if (method.expect) {
                    method.expect.call(jest, response, lastResponse);
                }

                lastResponse = response;
                done();
            } catch (error) {
                done(error);
            }
        });
    });

});
